@@ -27,6 +27,7 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib-object.h>
+#include <libebook/e-contact.h>
 #include <gnokii.h>
 
 #include <bluetooth/bluetooth.h>
@@ -53,29 +54,34 @@ phonemgr_utils_gn_statemachine_clear (struct gn_statemachine *state)
 const char *
 phonemgr_utils_gn_error_to_string (gn_error error, PhoneMgrError *perr)
 {
-	g_return_val_if_fail (perr != NULL, NULL);
-	*perr = -1;
+	if (perr != NULL)
+		*perr = -1;
 
 	switch (error) {
 	/* General codes */
 	case GN_ERR_NONE:
 		return NULL;
 	case GN_ERR_FAILED:
-		*perr = PHONEMGR_ERROR_COMMAND_FAILED;
+		if (perr != NULL)
+			*perr = PHONEMGR_ERROR_COMMAND_FAILED;
 		return "Command failed";
 	case GN_ERR_UNKNOWNMODEL:
-		*perr = PHONEMGR_ERROR_UNKNOWN_MODEL;
+		if (perr != NULL)
+			*perr = PHONEMGR_ERROR_UNKNOWN_MODEL;
 		return "Model specified isn't known/supported.";
 	case GN_ERR_INVALIDSECURITYCODE:
 		return "Invalid Security code.";
 	case GN_ERR_INTERNALERROR:
-		*perr = PHONEMGR_ERROR_INTERNAL_ERROR;
+		if (perr != NULL)
+			*perr = PHONEMGR_ERROR_INTERNAL_ERROR;
 		return "Problem occured internal to model specific code.";
 	case GN_ERR_NOTIMPLEMENTED:
-		*perr = PHONEMGR_ERROR_NOT_IMPLEMENTED;
+		if (perr != NULL)
+			*perr = PHONEMGR_ERROR_NOT_IMPLEMENTED;
 		return "Command called isn't implemented in model.";
 	case GN_ERR_NOTSUPPORTED:
-		*perr = PHONEMGR_ERROR_NOT_SUPPORTED;
+		if (perr != NULL)
+			*perr = PHONEMGR_ERROR_NOT_SUPPORTED;
 		return "Function not supported by the phone";
 	case GN_ERR_USERCANCELED:
 		return "User aborted the action.";
@@ -85,17 +91,20 @@ phonemgr_utils_gn_error_to_string (gn_error error, PhoneMgrError *perr)
 		return "The specified memory is full.";
 	/* Statemachine */
 	case GN_ERR_NOLINK:
-		*perr = PHONEMGR_ERROR_NO_LINK;
+		if (perr != NULL)
+			*perr = PHONEMGR_ERROR_NO_LINK;
 		return "Couldn't establish link with phone.";
 	case GN_ERR_TIMEOUT:
-		*perr = PHONEMGR_ERROR_TIME_OUT;
+		if (perr != NULL)
+			*perr = PHONEMGR_ERROR_TIME_OUT;
 		return "Command timed out.";
 	case GN_ERR_TRYAGAIN:
 		return "Try again.";
 	case GN_ERR_WAITING:
 		return "Waiting for the next part of the message.";
 	case GN_ERR_NOTREADY:
-		*perr = PHONEMGR_ERROR_NOT_READY;
+		if (perr != NULL)
+			*perr = PHONEMGR_ERROR_NOT_READY;
 		return "Device not ready.";
 	case GN_ERR_BUSY:
 		return "Command is still being executed.";
@@ -110,8 +119,10 @@ phonemgr_utils_address_is (const char *addr)
 	struct stat buf;
 
 	if (g_stat (addr, &buf) == 0 && S_ISCHR (buf.st_mode)) {
-		//FIXME this could also be IrDA
-		return PHONEMGR_CONNECTION_SERIAL;
+		if (strstr (addr, "ircomm") == NULL)
+			return PHONEMGR_CONNECTION_SERIAL;
+		else
+			return PHONEMGR_CONNECTION_IRDA;
 	} else if (bachk (addr) == 0) {
 		return PHONEMGR_CONNECTION_BLUETOOTH;
 	} else {
@@ -283,6 +294,11 @@ phonemgr_utils_write_config (const char *driver, const char *addr, int channel)
 					"port = %s\n"
 					"model = %s\n"
 					"connection = dku2libusb\n", addr, driver);
+	} else if (type == PHONEMGR_CONNECTION_IRDA) {
+		return g_strdup_printf ("[global]\n"
+					"port = %s\n"
+					"model = %s\n"
+					"connection = irda\n", addr, driver);
 	} else {
 		return g_strdup_printf ("[global]\n"
 			"port = %s\n"
@@ -516,6 +532,7 @@ phonemgr_utils_connect (const char *device, const char *driver, int channel, gbo
 	}
 	g_strfreev (lines);
 
+	memset (&data, 0, sizeof (data));
 	phonemgr_utils_gn_statemachine_clear (&state);
 
 	if (gn_cfg_phone_load("", &state) < 0) {
@@ -531,7 +548,7 @@ phonemgr_utils_connect (const char *device, const char *driver, int channel, gbo
 		return NULL;
 	}
 
-	phone_state = g_new (PhonemgrState, 1);
+	phone_state = g_new0 (PhonemgrState, 1);
 	phone_state->data = data;
 	phone_state->state = state;
 
@@ -712,3 +729,60 @@ phonemgr_utils_connection_is_supported (PhonemgrConnectionType type)
 	return gn_lib_is_connectiontype_supported (conntype);
 }
 
+#define SET_ENTRY(field, string) {				\
+	const char *s;						\
+	s = e_contact_get_const (contact, field);		\
+	if (s)							\
+		strncpy (string, s, sizeof (string));	\
+}
+
+#define SET_SUB_ENTRY(field, type) {						\
+	if (entry->subentries_count < GN_PHONEBOOK_SUBENTRIES_MAX_NUMBER) {	\
+		entry->subentries[entry->subentries_count].entry_type = type;	\
+		SET_ENTRY (field, entry->subentries[entry->subentries_count++].data.number); \
+	}									\
+}
+
+gboolean
+vcard_to_phonebook_entry (const char *vcard, gn_phonebook_entry *entry)
+{
+	EContact *contact;
+
+	contact = e_contact_new_from_vcard (vcard);
+	SET_ENTRY(E_CONTACT_FULL_NAME, entry->name);
+	if (entry->name == NULL)
+		return FALSE;
+
+	SET_ENTRY (E_CONTACT_PHONE_PRIMARY, entry->number);
+	if (entry->number == NULL)
+		SET_ENTRY(E_CONTACT_PHONE_HOME, entry->number);
+	if (entry->number == NULL)
+		SET_ENTRY(E_CONTACT_PHONE_MOBILE, entry->number);
+	if (entry->number == NULL)
+		return FALSE;
+
+	SET_ENTRY(E_CONTACT_FAMILY_NAME, entry->person.family_name);
+	SET_ENTRY(E_CONTACT_GIVEN_NAME, entry->person.given_name);
+	SET_ENTRY(E_CONTACT_TITLE, entry->person.honorific_prefixes);
+	SET_SUB_ENTRY (E_CONTACT_HOMEPAGE_URL, GN_PHONEBOOK_ENTRY_URL);
+	if (entry->subentries_count == 0)
+		SET_SUB_ENTRY (E_CONTACT_BLOG_URL, GN_PHONEBOOK_ENTRY_URL);
+	SET_SUB_ENTRY (E_CONTACT_EMAIL_1, GN_PHONEBOOK_ENTRY_Email);
+	SET_SUB_ENTRY (E_CONTACT_EMAIL_2, GN_PHONEBOOK_ENTRY_Email);
+	SET_SUB_ENTRY (E_CONTACT_EMAIL_3, GN_PHONEBOOK_ENTRY_Email);
+	SET_SUB_ENTRY (E_CONTACT_EMAIL_4, GN_PHONEBOOK_ENTRY_Email);
+	SET_SUB_ENTRY (E_CONTACT_ADDRESS_LABEL_HOME, GN_PHONEBOOK_ENTRY_Postal);
+	SET_SUB_ENTRY (E_CONTACT_ADDRESS_LABEL_WORK, GN_PHONEBOOK_ENTRY_Postal);
+	SET_SUB_ENTRY (E_CONTACT_ADDRESS_LABEL_OTHER, GN_PHONEBOOK_ENTRY_Postal);
+	SET_SUB_ENTRY (E_CONTACT_NOTE, GN_PHONEBOOK_ENTRY_Note);
+	SET_SUB_ENTRY (E_CONTACT_PHONE_BUSINESS, GN_PHONEBOOK_NUMBER_Work);
+	SET_SUB_ENTRY (E_CONTACT_PHONE_BUSINESS_2, GN_PHONEBOOK_NUMBER_Work);
+	SET_SUB_ENTRY (E_CONTACT_PHONE_HOME, GN_PHONEBOOK_NUMBER_Home);
+	SET_SUB_ENTRY (E_CONTACT_PHONE_HOME_2, GN_PHONEBOOK_NUMBER_Home);
+	SET_SUB_ENTRY (E_CONTACT_PHONE_MOBILE, GN_PHONEBOOK_NUMBER_Mobile);
+	SET_SUB_ENTRY (E_CONTACT_PHONE_BUSINESS_FAX, GN_PHONEBOOK_NUMBER_Fax);
+	SET_SUB_ENTRY (E_CONTACT_PHONE_HOME_FAX, GN_PHONEBOOK_NUMBER_Fax);
+
+	return TRUE;
+}
+
